@@ -280,7 +280,7 @@ static void install_anniversary_watch(void)
 
 static GtkWidget *wrap(GtkWidget *inner, gchar *label)
 {
-	GtkWidget *table = gtk_table_new(3, 2, FALSE);
+	GtkWidget *grid = gtk_grid_new();
 	GtkWidget *i;
 	GtkWidget *title;
 	GdkPixbuf *pixbuf;
@@ -293,24 +293,26 @@ static GtkWidget *wrap(GtkWidget *inner, gchar *label)
 	title = gtk_label_new(label);
 	gtk_label_set_use_markup(GTK_LABEL(title), TRUE);
 	gtk_misc_set_alignment(GTK_MISC(title), 0, 0.5);
-
-	gtk_table_attach(GTK_TABLE(table), i, 0, 1, 0, 1,
-				GTK_FILL, 0, 10, 0);
-	gtk_table_attach(GTK_TABLE(table), title, 1, 2, 0, 1,
-				GTK_FILL|GTK_EXPAND, 0, 10, 0);
-	gtk_table_attach(GTK_TABLE(table), gtk_hseparator_new(), 0, 2 ,1, 2,
-				GTK_FILL|GTK_EXPAND, 0, 0, 10);
-	gtk_table_attach(GTK_TABLE(table), inner, 1, 2, 2, 3,
-				GTK_FILL, 0, 10, 0);
-	gtk_widget_show_all(table);
-	return table;
+	gtk_grid_set_row_spacing(GTK_GRID(grid), 10);
+	gtk_grid_set_row_spacing(GTK_GRID(grid), 10);
+	gtk_grid_set_column_spacing(GTK_GRID(grid), 10);
+	gtk_grid_attach(GTK_GRID(grid), i, 0, 0, 1, 1);
+	gtk_grid_attach(GTK_GRID(grid), title, 1, 0, 1, 1);
+	gtk_grid_attach(GTK_GRID(grid), gtk_separator_new(GTK_ORIENTATION_HORIZONTAL), 0, 1 ,2, 1);
+	gtk_grid_attach(GTK_GRID(grid), inner, 0, 2, 2, 1);
+	gtk_widget_set_hexpand (grid, TRUE);
+	gtk_widget_set_hexpand (title, TRUE);
+	gtk_widget_set_halign (inner, GTK_ALIGN_CENTER);
+	gtk_widget_set_margin_left(i, 10);
+	gtk_widget_show_all(grid);
+	return grid;
 }
 
 void show_config(gpointer object, gpointer user_data)
 {
 	static GtkWidget *dialog = NULL;
 	GtkWidget *box;
-	GtkWidget *table;
+	GtkWidget *grid;
 	GtkWidget *label;
 	int row = 0;
 	GtkWidget *hbox;
@@ -339,14 +341,14 @@ void show_config(gpointer object, gpointer user_data)
 			
 	box = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
 	
-	table = gtk_table_new(3, 2, FALSE);
+	grid = gtk_grid_new();
 	
-	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
-	gtk_table_set_col_spacings(GTK_TABLE(table), 3);
+	gtk_grid_set_row_spacing(GTK_GRID(grid), 2);
+	gtk_grid_set_column_spacing(GTK_GRID(grid), 3);
 	
 	label = gtk_label_new(_("I quit on"));
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, row, 1, 1);
 	
 	stop_date_calendar = gtk_calendar_new();
 	gtk_calendar_select_month(GTK_CALENDAR(stop_date_calendar),
@@ -354,43 +356,43 @@ void show_config(gpointer object, gpointer user_data)
 					y + 1900);
 	gtk_calendar_select_day(GTK_CALENDAR(stop_date_calendar),
 					d);
-	gtk_table_attach_defaults(GTK_TABLE(table), stop_date_calendar, 1, 2, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), stop_date_calendar, 1, row, 1, 1);
 	
 	row++;
 	
 	label = gtk_label_new(_("I smoked"));
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, row, 1, 1);
 	
 	num_per_day_adj = GTK_ADJUSTMENT(
 		gtk_adjustment_new(num_per_day, 1.0, 100.0, 1.0, 1.0, 0.0));
 	num_per_day_spin = gtk_spin_button_new(num_per_day_adj, 1.0, 0);
 	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(num_per_day_spin), TRUE);
-	hbox = gtk_hbox_new(FALSE, 2);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
 	label = gtk_label_new(_("cigarettes per day"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 	
 	gtk_box_pack_start(GTK_BOX(hbox), num_per_day_spin, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
-	gtk_table_attach_defaults(GTK_TABLE(table), hbox, 1, 2, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), hbox, 1, row, 1, 1);
 	
 	row++;
 	
 	label = gtk_label_new(_("Each contained"));
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, row, 1, 1);
 	
 	nicotine_per_cig_adj = GTK_ADJUSTMENT(
 		gtk_adjustment_new(nicotine_per_cig, 0.1, 3.0, 0.1, 0.1, 0.0));
 	nicotine_per_cig_spin = gtk_spin_button_new(nicotine_per_cig_adj, 0.1, 1);
 	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(nicotine_per_cig_spin), TRUE);
-	hbox = gtk_hbox_new(FALSE, 2);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
 	label = gtk_label_new(_("mg of nicotine"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 	
 	gtk_box_pack_start(GTK_BOX(hbox), nicotine_per_cig_spin, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
-	gtk_table_attach_defaults(GTK_TABLE(table), hbox, 1, 2, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), hbox, 1, row, 1, 1);
 	
 	row++;
 	
@@ -398,21 +400,21 @@ void show_config(gpointer object, gpointer user_data)
 		gtk_adjustment_new(tar_per_cig, 1.0, 20.0, 0.5, 0.5, 0.0));
 	tar_per_cig_spin = gtk_spin_button_new(tar_per_cig_adj, 0.5, 1);
 	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(tar_per_cig_spin), TRUE);
-	hbox = gtk_hbox_new(FALSE, 2);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
 	label = gtk_label_new(_("mg of tar"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 	
 	gtk_box_pack_start(GTK_BOX(hbox), tar_per_cig_spin, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
-	gtk_table_attach_defaults(GTK_TABLE(table), hbox, 1, 2, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), hbox, 1, row, 1, 1);
 	
 	row++;
 	
 	label = gtk_label_new(_("It cost"));
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, row, 1, 1);
 	
-	hbox = gtk_hbox_new(FALSE, 2);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
 	price_per_pack_adj = GTK_ADJUSTMENT(
 		gtk_adjustment_new(price_per_pack, 0.1, 1000.0, 0.1, 0.1, 0.0));
 	price_per_pack_spin = gtk_spin_button_new(price_per_pack_adj, 0.1, 2);
@@ -434,10 +436,10 @@ void show_config(gpointer object, gpointer user_data)
 	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(pack_size_spin), TRUE);
 	
 	gtk_box_pack_start(GTK_BOX(hbox), pack_size_spin, TRUE, TRUE, 0);
-	gtk_table_attach_defaults(GTK_TABLE(table), hbox, 1, 2, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), hbox, 1, row, 1, 1);
 	
-	gtk_widget_show_all(table);
-	gtk_box_pack_start(GTK_BOX(box), wrap(table, _("<span size='x-large'><b>QuitCount - configuration</b></span>")), 
+	gtk_widget_show_all(grid);
+	gtk_box_pack_start(GTK_BOX(box), wrap(grid, _("<span size='x-large'><b>QuitCount - configuration</b></span>")), 
 			FALSE, FALSE, 10);
 	gtk_widget_set_size_request(dialog, 500, -1);
 	gtk_dialog_run(GTK_DIALOG(dialog));
@@ -468,7 +470,10 @@ void show_config(gpointer object, gpointer user_data)
 
 static void do_quit(gpointer object, gpointer user_data)
 {
-	gtk_main_quit();
+	GtkApplication *app = GTK_APPLICATION(user_data);
+	GList *windows = gtk_application_get_windows(app);
+	if (windows)
+		gtk_application_remove_window(app, GTK_WINDOW(windows->data));
 }
 
 static void do_about(gpointer object, gpointer user_data)
@@ -480,10 +485,14 @@ static void do_about(gpointer object, gpointer user_data)
 				  NULL};
 	const gchar *authors[] = {"Colin Leroy <colin@colino.net>", NULL};
 	
+	gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(dialog), "QuitCount");
+	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(dialog), VERSION);
+	if (icon != NULL)
+		gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(dialog), gtk_status_icon_get_pixbuf(icon));
 	gtk_about_dialog_set_artists(GTK_ABOUT_DIALOG(dialog), (const gchar **)artists);
 	gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(dialog), (const gchar **)authors);
 	gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(dialog), 
-		_("Copyright (C) 2009, Colin Leroy <colin@colino.net>\nGPL version 3 or later"));
+		_("Copyright (C) 2009-2013, Colin Leroy <colin@colino.net>\nGPL version 3 or later"));
 	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(dialog), 
 		"http://quitcount.sf.net/");
 	gtk_dialog_run(GTK_DIALOG(dialog));
@@ -498,39 +507,44 @@ static void do_activate(gpointer object, gpointer user_data)
 	int num_cig = num_day * num_per_day;
 	float nicotine, tar, life;
 	const char *n_unit, *t_unit, *l_unit;
-	GtkWidget *dialog = gtk_dialog_new_with_buttons(
+	gint response;
+	GtkWidget *grid, *label;
+	gint row = 0;
+	GtkWidget *box;
+	static GtkWidget *dialog = NULL;
+
+	if (dialog != NULL) {
+		gtk_dialog_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
+		return;
+	}
+
+	dialog = gtk_dialog_new_with_buttons(
 			_("QuitCount statistics"),
 			NULL, GTK_DIALOG_MODAL,
 			GTK_STOCK_PREFERENCES, QUITCOUNT_RESPONSE_PREFERENCES,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
 			NULL);
-	gint response;
 
-	GtkWidget *box = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
-	GtkWidget *table, *label;
-	gint row = 0;
+	box = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
 
 	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
 
-	if (num_day > 7)
-		table = gtk_table_new(6, 2, FALSE);
-	else
-		table = gtk_table_new(5, 2, FALSE);
+	grid = gtk_grid_new();
 	
-	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
-	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
+	gtk_grid_set_row_spacing(GTK_GRID(grid), 2);
+	gtk_grid_set_column_spacing(GTK_GRID(grid), 5);
 	
 	buf = g_strdup_printf("<span size='large'><b>%d</b></span>", num_day);
 	label = gtk_label_new(buf);
 	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
 	g_free(buf);
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, row, 1, 1);
 	
 	label = gtk_label_new(_("<span size='large'>days since I quit</span>"));
 	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), label, 1, row, 1, 1);
 	
 	row++;
 	
@@ -566,7 +580,7 @@ static void do_activate(gpointer object, gpointer user_data)
 		g_free(mfmt);
 		g_free(wfmt);
 		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-		gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, row, row+1);
+		gtk_grid_attach(GTK_GRID(grid), label, 1, row, 1, 1);
 
 		row++;
 
@@ -578,12 +592,12 @@ static void do_activate(gpointer object, gpointer user_data)
 	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
 	g_free(buf);
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, row, 1, 1);
 	
 	label = gtk_label_new(_("<span size='large'>cigarettes not smoked</span>"));
 	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), label, 1, row, 1, 1);
 	
 	row++;
 	
@@ -614,7 +628,7 @@ static void do_activate(gpointer object, gpointer user_data)
 	g_free(buf);
 	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), label, 1, row, 1, 1);
 	
 	row++;
 	
@@ -623,14 +637,14 @@ static void do_activate(gpointer object, gpointer user_data)
 	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
 	g_free(buf);
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, row, 1, 1);
 	
 	buf = g_strdup_printf(_("<span size='large'>%s saved</span>"), currency);
 	label = gtk_label_new(buf);
 	g_free(buf);
 	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), label, 1, row, 1, 1);
 	
 	row++;
 	
@@ -658,21 +672,22 @@ static void do_activate(gpointer object, gpointer user_data)
 	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
 	g_free(buf);
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, row, 1, 1);
 	
 	label = gtk_label_new(_("<span size='large'>of life expectancy gained</span>"));
 	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, row, row+1);
+	gtk_grid_attach(GTK_GRID(grid), label, 1, row, 1, 1);
 	
 	row++;
 	
-	gtk_widget_show_all(table);
-	gtk_box_pack_start(GTK_BOX(box), wrap(table, 
+	gtk_widget_show_all(grid);
+	gtk_box_pack_start(GTK_BOX(box), wrap(grid, 
 		_("<span size='x-large'><b>QuitCount - statistics</b></span>")), FALSE, FALSE, 10);
 	gtk_widget_set_size_request(dialog, 500, -1);
 	response = gtk_dialog_run(GTK_DIALOG(dialog));
 	gtk_widget_destroy(dialog);
+	dialog = NULL;
 
 	if (response == QUITCOUNT_RESPONSE_PREFERENCES)
 		show_config(NULL, NULL);
@@ -687,6 +702,7 @@ static void do_popup_menu(GtkStatusIcon *i, guint button, guint activate_time,
 	gchar *path = g_strconcat(ICONSDIR, G_DIR_SEPARATOR_S, PACKAGE, ".svg", NULL);
 	gint w,h;
 	GdkPixbuf *pixbuf;
+	GtkApplication *app = GTK_APPLICATION(user_data);
 	
 	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
 
@@ -725,7 +741,7 @@ static void do_popup_menu(GtkStatusIcon *i, guint button, guint activate_time,
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 	gtk_widget_show(item);
 	g_signal_connect(G_OBJECT(item), "activate", 
-			 G_CALLBACK(do_quit), NULL);
+			 G_CALLBACK(do_quit), app);
 	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, gtk_status_icon_position_menu, i,
 			button, activate_time);
 }
@@ -747,9 +763,9 @@ void run_quitcount(GtkApplication *app)
 		install_anniversary_watch();
 	
 		g_signal_connect(G_OBJECT(icon), "popup-menu", 
-				 G_CALLBACK(do_popup_menu), NULL);
+				 G_CALLBACK(do_popup_menu), app);
 		g_signal_connect(G_OBJECT(icon), "activate", 
-				 G_CALLBACK(do_activate), NULL);
+				 G_CALLBACK(do_activate), app);
 	} else if (stop_date != (time_t)0) {
 		do_activate(NULL, NULL);
 	}
